@@ -1,6 +1,6 @@
 //! # Examples
 //!
-//! Spawn a process outside of the sandbox, only works in a Flatpak.
+//! Spawn a process inside of the sandbox, only works in a Flatpak.
 //!
 //! ```rust,no_run
 //! use std::collections::HashMap;
@@ -28,7 +28,7 @@
 use std::{
     collections::HashMap,
     fmt::Debug,
-    os::unix::prelude::{AsRawFd, RawFd},
+    os::fd::{BorrowedFd, OwnedFd},
     path::Path,
 };
 
@@ -36,7 +36,7 @@ use enumflags2::{bitflags, BitFlags};
 use futures_util::Stream;
 use serde::Serialize;
 use serde_repr::{Deserialize_repr, Serialize_repr};
-use zbus::zvariant::{Fd, OwnedObjectPath, SerializeDict, Type};
+use zbus::zvariant::{self, Fd, OwnedObjectPath, SerializeDict, Type};
 
 use crate::{proxy::Proxy, Error, FilePath};
 
@@ -116,11 +116,11 @@ pub struct SpawnOptions {
     /// A list of file descriptor for files inside the sandbox that will be
     /// exposed to the new sandbox, for reading and writing.
     #[zvariant(rename = "sandbox-expose-fd")]
-    sandbox_expose_fd: Option<Vec<Fd>>,
+    sandbox_expose_fd: Option<Vec<zvariant::OwnedFd>>,
     /// A list of file descriptor for files inside the sandbox that will be
     /// exposed to the new sandbox, read-only.
     #[zvariant(rename = "sandbox-expose-fd-ro")]
-    sandbox_expose_fd_ro: Option<Vec<Fd>>,
+    sandbox_expose_fd_ro: Option<Vec<zvariant::OwnedFd>>,
     /// Flags affecting the created sandbox.
     #[zvariant(rename = "sandbox-flags")]
     sandbox_flags: Option<BitFlags<SandboxFlags>>,
@@ -130,11 +130,11 @@ pub struct SpawnOptions {
     /// A file descriptor of the directory that  will be used as `/usr` in the
     /// new sandbox.
     #[zvariant(rename = "usr-fd")]
-    usr_fd: Option<RawFd>,
+    usr_fd: Option<zvariant::OwnedFd>,
     /// A file descriptor of the directory that  will be used as `/app` in the
     /// new sandbox.
     #[zvariant(rename = "app-fd")]
-    app_fd: Option<RawFd>,
+    app_fd: Option<zvariant::OwnedFd>,
 }
 
 impl SpawnOptions {
@@ -167,26 +167,26 @@ impl SpawnOptions {
 
     /// Sets the list of file descriptors of files to expose the new sandbox.
     #[must_use]
-    pub fn sandbox_expose_fd<P: IntoIterator<Item = I>, I: AsRawFd + Type + Serialize>(
+    pub fn sandbox_expose_fd<P: IntoIterator<Item = OwnedFd>>(
         mut self,
         sandbox_expose_fd: impl Into<Option<P>>,
     ) -> Self {
         self.sandbox_expose_fd = sandbox_expose_fd
             .into()
-            .map(|a| a.into_iter().map(|s| Fd::from(s.as_raw_fd())).collect());
+            .map(|a| a.into_iter().map(zvariant::OwnedFd::from).collect());
         self
     }
 
     /// Sets the list of file descriptors of files to expose the new sandbox,
     /// read-only.
     #[must_use]
-    pub fn sandbox_expose_fd_ro<P: IntoIterator<Item = I>, I: AsRawFd + Type + Serialize>(
+    pub fn sandbox_expose_fd_ro<P: IntoIterator<Item = OwnedFd>>(
         mut self,
         sandbox_expose_fd_ro: impl Into<Option<P>>,
     ) -> Self {
         self.sandbox_expose_fd_ro = sandbox_expose_fd_ro
             .into()
-            .map(|a| a.into_iter().map(|s| Fd::from(s.as_raw_fd())).collect());
+            .map(|a| a.into_iter().map(zvariant::OwnedFd::from).collect());
         self
     }
 
@@ -215,16 +215,16 @@ impl SpawnOptions {
     /// Set a file descriptor of the directory that  will be used as `/usr` in
     /// the new sandbox.
     #[must_use]
-    pub fn usr_fd<F: AsRawFd>(mut self, fd: impl Into<Option<F>>) -> Self {
-        self.usr_fd = fd.into().map(|s| s.as_raw_fd());
+    pub fn usr_fd(mut self, fd: impl Into<Option<OwnedFd>>) -> Self {
+        self.usr_fd = fd.into().map(|f| f.into());
         self
     }
 
     /// Set a file descriptor of the directory that  will be used as `/app` in
     /// the new sandbox.
     #[must_use]
-    pub fn app_fd<F: AsRawFd>(mut self, fd: impl Into<Option<F>>) -> Self {
-        self.app_fd = fd.into().map(|s| s.as_raw_fd());
+    pub fn app_fd(mut self, fd: impl Into<Option<OwnedFd>>) -> Self {
+        self.app_fd = fd.into().map(|f| f.into());
         self
     }
 }
@@ -240,7 +240,7 @@ struct CreateMonitorOptions {}
 /// sandbox. For example, it allows you to restart the applications or start a
 /// more sandboxed instance.
 ///
-/// Wrapper of the DBus interface: [`org.freedesktop.portal.Flatpak`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-org.freedesktop.portal.Flatpak).
+/// Wrapper of the DBus interface: [`org.freedesktop.portal.Flatpak`](https://docs.flatpak.org/en/latest/portal-api-reference.html#gdbus-org.freedesktop.portal.Flatpak).
 #[derive(Debug)]
 #[doc(alias = "org.freedesktop.portal.Flatpak")]
 pub struct Flatpak<'a>(Proxy<'a>);
@@ -256,22 +256,31 @@ impl<'a> Flatpak<'a> {
     /// when an update for the caller becomes available, and can be used to
     /// install it.
     ///
+    /// # Required version
+    ///
+    /// The method requires the 2nd version implementation of the portal and
+    /// would fail with [`Error::RequiresVersion`] otherwise.
+    ///
     /// # Specifications
     ///
-    /// See also [`CreateUpdateMonitor`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-Flatpak.CreateUpdateMonitor).
+    /// See also [`CreateUpdateMonitor`](https://docs.flatpak.org/en/latest/portal-api-reference.html#gdbus-method-org-freedesktop-portal-Flatpak.CreateUpdateMonitor).
     #[doc(alias = "CreateUpdateMonitor")]
     #[doc(alias = "xdp_portal_update_monitor_start")]
     pub async fn create_update_monitor(&self) -> Result<UpdateMonitor<'a>, Error> {
         let options = CreateMonitorOptions::default();
         let path = self
             .0
-            .call::<OwnedObjectPath>("CreateUpdateMonitor", &(options))
+            .call_versioned::<OwnedObjectPath>("CreateUpdateMonitor", &(options), 2)
             .await?;
 
         UpdateMonitor::new(path.into_inner()).await
     }
 
     /// Emitted when a process starts by [`spawn()`][`Flatpak::spawn`].
+    ///
+    /// # Specifications
+    ///
+    /// See also [`SpawnStarted`](https://docs.flatpak.org/en/latest/portal-api-reference.html#gdbus-signal-org-freedesktop-portal-Flatpak.SpawnStarted).
     #[doc(alias = "SpawnStarted")]
     pub async fn receive_spawn_started(&self) -> Result<impl Stream<Item = (u32, u32)>, Error> {
         self.0.signal("SpawnStarted").await
@@ -282,10 +291,10 @@ impl<'a> Flatpak<'a> {
     ///
     /// # Specifications
     ///
-    /// See also [`SpawnExited`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-signal-org-freedesktop-portal-Flatpak.SpawnExited).
+    /// See also [`SpawnExited`](https://docs.flatpak.org/en/latest/portal-api-reference.html#gdbus-signal-org-freedesktop-portal-Flatpak.SpawnExited).
     #[doc(alias = "SpawnExited")]
     #[doc(alias = "XdpPortal::spawn-exited")]
-    pub async fn receive_spawn_existed(&self) -> Result<impl Stream<Item = (u32, u32)>, Error> {
+    pub async fn receive_spawn_exited(&self) -> Result<impl Stream<Item = (u32, u32)>, Error> {
         self.0.signal("SpawnExited").await
     }
 
@@ -309,14 +318,14 @@ impl<'a> Flatpak<'a> {
     ///
     /// # Specifications
     ///
-    /// See also [`Spawn`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-Flatpak.Spawn).
+    /// See also [`Spawn`](https://docs.flatpak.org/en/latest/portal-api-reference.html#gdbus-method-org-freedesktop-portal-Flatpak.Spawn).
     #[doc(alias = "Spawn")]
     #[doc(alias = "xdp_portal_spawn")]
     pub async fn spawn(
         &self,
         cwd_path: impl AsRef<Path>,
         argv: &[impl AsRef<Path>],
-        fds: HashMap<u32, Fd>,
+        fds: HashMap<u32, BorrowedFd<'_>>,
         envs: HashMap<&str, &str>,
         flags: BitFlags<SpawnFlags>,
         options: SpawnOptions,
@@ -326,6 +335,7 @@ impl<'a> Flatpak<'a> {
             .iter()
             .map(FilePath::new)
             .collect::<Result<Vec<FilePath>, _>>()?;
+        let fds: HashMap<u32, Fd> = fds.iter().map(|(k, val)| (*k, Fd::from(val))).collect();
         self.0
             .call("Spawn", &(cwd_path, argv, fds, envs, flags, options))
             .await
@@ -342,7 +352,7 @@ impl<'a> Flatpak<'a> {
     ///
     /// # Specifications
     ///
-    /// See also [`SpawnSignal`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-method-org-freedesktop-portal-Flatpak.SpawnSignal).
+    /// See also [`SpawnSignal`](https://docs.flatpak.org/en/latest/portal-api-reference.html#gdbus-method-org-freedesktop-portal-Flatpak.SpawnSignal).
     #[doc(alias = "SpawnSignal")]
     #[doc(alias = "xdp_portal_spawn_signal")]
     pub async fn spawn_signal(
@@ -360,12 +370,26 @@ impl<'a> Flatpak<'a> {
     ///
     /// # Specifications
     ///
-    /// See also [`supports`](https://flatpak.github.io/xdg-desktop-portal/index.html#gdbus-property-org-freedesktop-portal-Flatpak.supports).
+    /// See also [`supports`](https://docs.flatpak.org/en/latest/portal-api-reference.html#gdbus-property-org-freedesktop-portal-Flatpak.supports).
     pub async fn supports(&self) -> Result<BitFlags<SupportsFlags>, Error> {
-        self.0.property::<BitFlags<SupportsFlags>>("supports").await
+        self.0
+            .property_versioned::<BitFlags<SupportsFlags>>("supports", 3)
+            .await
+    }
+}
+
+impl<'a> std::ops::Deref for Flatpak<'a> {
+    type Target = zbus::Proxy<'a>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
     }
 }
 
 /// Monitor if there's an update it and install it.
 mod update_monitor;
 pub use update_monitor::{UpdateInfo, UpdateMonitor, UpdateProgress, UpdateStatus};
+
+/// Provide for a way to execute processes outside of the sandbox
+mod development;
+pub use development::{Development, HostCommandFlags};
